/**********************************************************
  CAT25512_EEPROM - NightShade_Treo by NightShade Electronics

  This sketch demonstrates the functionality of the
  NightShade Trēo CAT25512 EEPROM module. (NSE-1138-1) It 
  performs write  and read cycles, playing "telephone" with 
  the EEPROM.

  Created by Aaron D. Liebold
  on February 15, 2021

  Links:
  NightShade Trēo System: https://nightshade.net/treo
  Product Page: https://nightshade.net/product/treo-512kb-eeprom-cat25512/

  Distributed under the MIT license
  Copyright (C) 2021 NightShade Electronics
  https://opensource.org/licenses/MIT
**********************************************************/

#include <NightShade_Treo.h>

NightShade_Treo_CAT25512 eeprom(1, 10);

char buffer[128] =  "Only 128 bytes can be read or written to the CAT25512 EEPROM at a time. This is called a page write/read...................128B";

void setup() {
  Serial.begin(115200);
  Serial.print("Initial character string: \"");
  Serial.print(buffer);
  Serial.println('\"');
  eeprom.begin();
}

void loop() {
  Serial.println("\nWriting last string to memory...");
  eeprom.startMemoryWrite(0x00);
  for (int x = 0; x < 128; ++x) eeprom.write(buffer[x]);
  eeprom.endMemoryWrite();
  Serial.println("Clearing software buffer.");
  for (int x = 0; x < 128; ++x) buffer[x] = 0;
  Serial.println("Buffer cleared.");
  Serial.println("Reading memory.");
  eeprom.requestMemoryRead(0x00, 128);
  for (int x = 0; x < 128; ++x) buffer[x] = eeprom.read();
  Serial.println("String read from EEPROM:");
  for (int x = 0; x < 128; ++x) Serial.write(buffer[x]);
  Serial.println();

  delay(500);
}
