/**********************************************************
  HDC1080_TempHumidity - NightShade_Treo by NightShade Electronics

  This sketch demonstrates the functionality of the  
  NightShade Trēo HDC1080 temperature and humidity sensing 
  module. (NSE-1135-1) It prints the temperature and 
  humidity values to Serial at 115200 baudrate.
  
  Created by Aaron D. Liebold
  on February 15, 2021

  Links:
  NightShade Trēo System: https://nightshade.net/treo
  Product Page: https://nightshade.net/product/treo-temp-humidity-sensor-hdc1080/

  Distributed under the MIT license
  Copyright (C) 2021 NightShade Electronics
  https://opensource.org/licenses/MIT
**********************************************************/

// Include NightShade Treo Library
#include <NightShade_Treo.h>

NightShade_Treo_HDC1080 sensor(1);

unsigned long lastHeater = 0, lastUpdate = 0;

void setup() {
  sensor.begin();
  Serial.begin(115200);

  sensor.setTemperatureResolution(0);
  sensor.setHumidityResolution(0);

  Serial.print("Device ID: 0x");
  Serial.print(sensor.readDeviceId(), HEX);
  Serial.print("\nManufacturer ID: 0x");
  Serial.print(sensor.readManufacturerId(), HEX);
  Serial.print("\nSerial ID: 0x");
  Serial.print((unsigned long) sensor.readSerialId(), HEX);
  Serial.print("\n");

  delay(1);
}

void loop() {
  if (millis() >= lastUpdate + 500) {
    lastUpdate = millis();
    sensor.acquireData(1, 1);
    Serial.print(sensor.readTemperature());
    Serial.print("C\t");
    Serial.print(sensor.readHumidity());
    Serial.print("%\n");
  }
}
