/**********************************************************
  LIS3MDL_Magnetometer_Filtering - NightShade_Treo by NightShade Electronics

  This sketch demonstrates the functionality of the
  NightShade Trēo LIS3MDL magnetometer module. (NSE-1126-1) 
  It prints the magnetometer data, a calculated heading, 
  and temperature from the sensor and prints in out as 
  Serial at 115200 baudrate.

  Created by Aaron D. Liebold
  on February 15, 2021

  Links:
  NightShade Trēo System: https://nightshade.net/treo
  Product Page: https://nightshade.net/product/treo-3-axis-magnetometer-lis3mdl/

  Distributed under the MIT license
  Copyright (C) 2021 NightShade Electronics
  https://opensource.org/licenses/MIT
**********************************************************/

// Include NightShade Treo Library
#include <NightShade_Treo.h>
#include <math.h>

// Declare Objects
NightShade_Treo_LIS3MDL sensor(1);

#define FILTER_COEF 10
#define READ_FREQ   100
#define PRINT_FREQ  2

unsigned long lastRead, lastPrint;
float X, Y, Z, temp;

void setup() {
  sensor.begin();
  Serial.begin(115200);
}

void loop() {
  if (millis() >= lastRead + (1000 / READ_FREQ)) {
    sensor.acquireMagData();
    X = (((float) sensor.readX() / sensor.oneGaussValue()) + X*(FILTER_COEF - 1))/(FILTER_COEF);
    Y = (((float) sensor.readY() / sensor.oneGaussValue()) + Y*(FILTER_COEF - 1))/(FILTER_COEF);
    Z = (((float) sensor.readZ() / sensor.oneGaussValue()) + Z*(FILTER_COEF - 1))/(FILTER_COEF);
    temp = (((float) sensor.readTemp() / 10) + temp*(FILTER_COEF - 1))/(FILTER_COEF);
    lastRead = millis();
  }

  if (millis() >= lastPrint + (1000 / PRINT_FREQ)) {
    float azimuth =  atan2(Y, X) * 180 / M_PI;
    if (azimuth < 0) azimuth += 360.0;

    Serial.print("Az: ");
    Serial.print(azimuth);
    Serial.print("deg \t");
    Serial.print(X, 2);
    Serial.print("G\t");
    Serial.print(Y, 2);
    Serial.print("G\t");
    Serial.print(Z, 2);
    Serial.print("G\t");
    Serial.print(temp, 2);
    Serial.print("C\t\n");
    lastPrint = millis();
  }
}
